/*
 * Copyright (c) 1999, 2017, Oracle and/or its affiliates. All rights reserved.
 */

package $packageName$;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;
import java.util.concurrent.RunnableFuture;

import com.oracle.jmc.common.item.IAggregator;
import com.oracle.jmc.common.item.IItemCollection;
import com.oracle.jmc.common.item.ItemFilters;
import com.oracle.jmc.common.unit.UnitLookup;
import com.oracle.jmc.common.util.IPreferenceValueProvider;
import com.oracle.jmc.common.util.TypedPreference;
import com.oracle.jmc.flightrecorder.jdk.JdkAggregators;
import com.oracle.jmc.flightrecorder.jdk.JdkAttributes;
import com.oracle.jmc.flightrecorder.jdk.JdkFilters;
import com.oracle.jmc.flightrecorder.rules.IRule;
import com.oracle.jmc.flightrecorder.rules.Result;
import com.oracle.jmc.flightrecorder.rules.util.JfrRuleTopics;

/**
 * Simple JFR rule example. The rule will check the environment variable named JFR_RULE_TEST (by default) and 
 * try to parse the contents as a double to use as the score returned by the rule.
 * 
 * <p>The rule illustrates how to:
 *   <ul>
 *     <li>Create a rule result</li>
 *     <li>Use preferences in a rule</li>
 *     <li>How to filter an event collection (IItemCollection)</li>
 *     <li>How to calculate an aggregate on an event collection</li>
 *   </ul>
 *   
 * <p>Most commonly used aggregates are specified either in the Aggregators class (most commonly used numerical aggregates, such as sum, avg, stddev), 
 * or the JfrAggregators class.
 */
@SuppressWarnings("nls")
public class $className$ implements IRule {
	private static final TypedPreference<String> PREFERENCE_ENVIRONMENT_VARIABLE_NAME = 
		new TypedPreference<>("environmentVariable", "Environment Variable", "The name of the environment variable containing the floating point score", UnitLookup.PLAIN_TEXT.getPersister(), "JFR_RULE_TEST");
	private static final List<TypedPreference<?>> CONFIG_ATTRIBUTES = 
		Arrays.<TypedPreference<?>> asList(PREFERENCE_ENVIRONMENT_VARIABLE_NAME);

	private Result getResult(IItemCollection items, IPreferenceValueProvider valueProvider) {
		String variableName = valueProvider.getPreferenceValue(PREFERENCE_ENVIRONMENT_VARIABLE_NAME);
		String environmentVariableValue = getEnvironmentVariable(variableName, items);

		if (environmentVariableValue == null) {
			return new Result(this, 100, "Could not find the environment variable named " + variableName);
		}

		double score = 0;
		try {
			score = Double.parseDouble(environmentVariableValue);
		} catch (NumberFormatException e) {
			return new Result(this, 100, "Could not parse the value for the environment variable named " + variableName);
		}

		return new Result(this, score, "The result from parsing the information in the environment variable named " + variableName + " was " + score);
	}

	@Override
	public RunnableFuture<Result> evaluate(final IItemCollection items, final IPreferenceValueProvider valueProvider) {
		FutureTask<Result> evaluationTask = new FutureTask<>(new Callable<Result>() {
			@Override
			public Result call() throws Exception {
				return getResult(items, valueProvider);
			}
		});
		return evaluationTask;
	}

	private String getEnvironmentVariable(String variableName, IItemCollection items) {
		IItemCollection envItems = items.apply(ItemFilters.and(JdkFilters.ENVIRONMENT_VARIABLE, ItemFilters.equals(JdkAttributes.ENVIRONMENT_KEY, variableName)));
		IAggregator<String, ?> firstAggregator = JdkAggregators.first(JdkAttributes.ENVIRONMENT_VALUE);
		return envItems.getAggregate(firstAggregator);
	}

	@Override
	public Collection<TypedPreference<?>> getConfigurationAttributes() {
		return CONFIG_ATTRIBUTES;
	}

	@Override
	public String getId() {
		return "$packageName$.$className$";
	}

	@Override
	public String getName() {
		return "$ruleName$";
	}

	@Override
	public String getTopic() {
		return JfrRuleTopics.ENVIRONMENT_VARIABLES_TOPIC;
	}
}
